/**

	3D Billiards Demo
	
	File			:	Ball Class Code

	Created by		:	Reznichenko Anton aka AlterVision
						mailto:me@av13.ru
						http://www.av13.ru

	Project started :	14 may 2009 - 12:27
	Last edited		:	18 may 2009 - 19:29

*/

#include <math.h>
#include <stdlib.h>
#include <windows.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include "world.h"

//
// Materials :: Start
//

typedef struct _material_t {
	float ambient[4];
	float diffuse[4];
	float specular[4];
	float emission[4];
	float shininess;
} material_t;

#define MATERIALS 5
material_t materials[] = {
	// Gold
	{
		{0.24725f,	0.1995f,	0.0745f,	1.0f},
		{0.75164f,	0.60648f,	0.22648f,	1.0f},
		{0.628281f, 0.555802f,	0.366065f,	1.0f},
		{0.0f,		0.0f,		0.0f,		1.0f},
		83.2f
	},
	// Silver
	{
		{0.19225f,	0.19225f,	0.19225f,	1.0f},
		{0.50754f,	0.50754f,	0.50754f,	1.0f},
		{0.508273f, 0.508273f,	0.508273f,	1.0f},
		{0.0f,		0.0f,		0.0f,		1.0f},
		51.2f
	},
	// Emerald
	{
		{0.0215f,	0.1745f,	0.0215f,	1.0f},
		{0.07568f,	0.61424f,	0.07568f,	1.0f},
		{0.633f,	0.727811f,	0.633f,		1.0f},
		{0.0f,		0.0f,		0.0f,		1.0f},
		76.8f
	},
	// Ruby
	{
		{0.1745f,	0.01175f,	0.01175f,	1.0f},
		{0.61424f,	0.04136f,	0.04136f,	1.0f},
		{0.727811f, 0.626959f,	0.626959f,	1.0f},
		{0.0f,		0.0f,		0.0f,		1.0f},
		76.8f
	},
	// Box Material :: Chrome
	{
		{0.25f,		0.25f,		0.25f,		1.0f},
		{0.4f,		0.4f,		0.4f,		1.0f},
		{0.774597f, 0.774597f,	0.774597f, 1.0f},
		{0.0f,		0.0f,		0.0f,		1.0f},
		78.6f
	}
};

//
// Materials :: End
//

//
// Object and Memory Handling :: Start
//

// Constructor
c_ball::c_ball (int id)		{

	// Using Null Vector-Position 
	vp_t null_pos = {0.0, 0.0, 0.0};

	// Setting Up Default Variables
	this->id		= id;			// The ID
	this->active	= true;			// Object is Active
	this->mutex		= false;		// Object is Free

	this->position	= null_pos;		// Position in Center
	this->speed		= null_pos;		// Not Moving
	this->mass		= 1.0;			// With 1ed of Mass
	this->size		= 1.0;			// And 1ed of Size
	this->air		= 0.004;		// Air Resistance

	// Coloring
	this->material = rand () % MATERIALS;

}

// Destructor
c_ball::~c_ball ()	{ /* Silence is Golden */ }

// Creator (Constructor Extender)
int c_ball::create (void *vworld, vp_t position) {

	// Setting Start Position of the Object
	this->position = position;
	return true;

}

// Destroyer (Destructor Extender)
int c_ball::destroy (void *vworld) {

	// Die ... %)
	return true;

}

//
// Object and Memory Handling :: End
//

//
// Rendering :: Start
//

// Simple Rendering 
int c_ball::render  (void *vworld) {

	// Getting Quadrick Mesh Object
	c_world_p world = (c_world_p) vworld;
	GLUquadricObj *quadric = (GLUquadricObj*) world->quadric;

	// Setting Draw Coordinates
	glTranslatef(this->position.x, this->position.y, this->position.z);
	
	// Colors And Materials
	glColor3f (1.0, 1.0, 1.0);
	material_t mat = materials[this->material];
	glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,	mat.ambient);
	glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,	mat.diffuse);
	glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR,	mat.specular);
	glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION,	mat.emission);
	glMaterialf  (GL_FRONT_AND_BACK, GL_SHININESS,	mat.shininess);
	
	// Rendering
	gluSphere (quadric, this->size, 16, 16);

	// Resetting Coordinates
	glTranslatef(-this->position.x, -this->position.y, -this->position.z);

	// ... to say the truth ...
	return true;

}

//
// Rendering :: Start
//

/* Offtop: Force, Mass and Acceleration Will Be With You, Luke ...  */

//
// Physics Processing :: Start
//

// Processing Object Movement
int c_ball::process (void *vworld) {

	// Preparing New Position
	vp_t new_pos = this->position;

	// Adding Air Resistance to Speed
	this->speed.x -= this->speed.x * this->air;
	this->speed.y -= this->speed.y * this->air;
	this->speed.z -= this->speed.z * this->air;

	// New Position Creating
	new_pos.x += this->speed.x;
	new_pos.y += this->speed.y;
	new_pos.z += this->speed.z;

	// World Bounds
	float bound_x_min = (this->size / 2.0) - BOX_SIZE_X;
	float bound_x_max = BOX_SIZE_X - (this->size / 2.0);
	float bound_y_min = (this->size / 2.0) - BOX_SIZE_Y;
	float bound_y_max = BOX_SIZE_Y - (this->size / 2.0);
	float bound_z_min = (this->size / 2.0) - BOX_SIZE_Z;
	float bound_z_max = BOX_SIZE_Z - (this->size / 2.0);

	// Bounds - X - Make Collisions
	if (new_pos.x <= bound_x_min || new_pos.x >= bound_x_max) {
		this->speed.x = -this->speed.x;
		new_pos = this->position;
	}

	// Bounds - Y - Make Collisions
	if (new_pos.y <= bound_y_min || new_pos.y >= bound_y_max) {
		this->speed.y = -this->speed.y;
		new_pos = this->position;
	}

	// Bounds - Z - Make Collisions
	if (new_pos.z <= bound_z_min || new_pos.z >= bound_z_max) {
		this->speed.z = -this->speed.z;
		new_pos = this->position;
	}

	// Setting Up New Position
	this->position = new_pos;

	// ... to say the truth ...
	return true;

}

// Adding New Vecrot to Speed
int c_ball::vector (vp_t force) {

	// Simply Use Vector Summ of Speed and Force
	this->speed.x += force.x;
	this->speed.y += force.y;
	this->speed.z += force.z;

	// ... to say the truth ...
	return true;

}

// Processing Object Collisions
vp_t c_ball::collide(vp_t position, vp_t speed, float mass) {

	// Speed Before Collision
	vp_t speed1 = this->speed;
	vp_t speed2 = speed;

	//
	// Vector Collision Processing :: Woodoo Start
	//

	// Creating Collision Vector
	vp_t collision_vector = {
		this->position.x - position.x,
		this->position.y - position.y,
		this->position.z - position.z
	};

	// Normalize Collision Vector
	float collision_vector_l = sqrt (pow(collision_vector.x, 2) + pow(collision_vector.y, 2) + pow(collision_vector.z, 2));
	collision_vector.x /= collision_vector_l;
	collision_vector.y /= collision_vector_l;
	collision_vector.z /= collision_vector_l;

	// Normal Multipliers for Reaction Vectors
	float speed1reaction = mass / this->mass * sqrt (pow(speed2.x, 2) + pow(speed2.y, 2) + pow(speed2.z, 2));
	float speed2reaction = this->mass / mass * sqrt (pow(speed1.x, 2) + pow(speed1.y, 2) + pow(speed1.z, 2));

	// Preparing Speed Variables

	vp_t new_speed1 = {
		speed1.x + collision_vector.x * speed1reaction,
		speed1.y + collision_vector.y * speed1reaction,
		speed1.z + collision_vector.z * speed1reaction
	};

	vp_t new_speed2 = {
		speed2.x - collision_vector.x * speed2reaction,
		speed2.y - collision_vector.y * speed2reaction,
		speed2.z - collision_vector.z * speed2reaction
	};
	

	//
	// Vector Collision Processing :: Woodoo End
	//

	// Speed After Collision
	
	// Speed Before Collision Looses 90% of Power
	speed1.x *= 0.1;		speed1.y *= 0.1;		speed1.z *= 0.1;
	speed2.x *= 0.1;		speed2.y *= 0.1;		speed2.z *= 0.1;

	// Speed After Collision Looses 10% of Power
	new_speed1.x *= 0.9;	new_speed1.y *= 0.9;	new_speed1.z *= 0.9;
	new_speed2.x *= 0.9;	new_speed2.y *= 0.9;	new_speed2.z *= 0.9;

	// Vector Summing of New Object 1 Speed
	speed1.x += new_speed1.x; 
	speed1.y += new_speed1.y;
	speed1.z += new_speed1.z;
	
	// Vector Summing of New Object 2 Speed
	speed2.x += new_speed2.x; 
	speed2.y += new_speed2.y;
	speed2.z += new_speed2.z;

	this->speed = speed1;
	return speed2;

}

//
// Physics Processing :: End
//

/* Offtop: Force Will Be With You Always ...  */

//
// Thread Syncronization :: Start
//

// Locking the Object
void c_ball::lock() {
	
	// Waiting Until Object Gets Unlocked
	while (this->mutex) {
		// Silence is Still Golden ...
	}

	// Setting Mutex Value to True
	this->mutex = true;

}

// Unlocking the Object
void c_ball::unlock() {

	// Simply Resetting Mutex Value
	this->mutex = false;

}

//
// Thread Syncronization :: End
//

// end. =)
