/**

	3D Billiards Demo
	
	File			:	Interface Class Code

	Created by		:	Reznichenko Anton aka AlterVision
						mailto:me@av13.ru
						http://www.av13.ru

	Project started :	14 may 2009 - 12:27
	Last edited		:	18 may 2009 - 19:28

*/

#include <windows.h>
#include <math.h>
#include <gl\gl.h>                      
#include <gl\glu.h>    
#include "world.h"

c_iface::c_iface() {

	this->xa = 30.0;
	this->ya = 0.0;

	this->vxa = 30.0;
	this->vya = 0.0;

	this->xas = 0.0;
	this->yas = 0.0;

	this->vxs = 0.0;
	this->vys = 0.0;

	this->camera = 20.0;

	this->view = 0;

	this->view_id = 0;

}

c_iface::~c_iface() { }

float c_iface::wg2r (float angle) { // Gradus to Radian
	if (angle > 360.0)	angle -= 360.0;
	if (angle < 0.0)	angle += 360.0;
	return angle * 3.141 * 2.0 / 360.0;
}

int c_iface::process(void *vworld) {

	c_world_p world = (c_world_p) vworld;

	if (world->keys[VK_F1]) this->view = 0;
	if (world->keys[VK_F2]) this->view = 1;

	switch (this->view) {

		case 0:

			if (world->keys[VK_NEXT])	this->camera += 1.0;
			if (world->keys[VK_PRIOR])	this->camera -= 1.0;
			if (this->camera < 10.0)	this->camera = 10.0;
			if (this->camera > 40.0)	this->camera = 40.0;

			// Rotation
			if ( world->keys[VK_UP] )	{
				this->xas += (XROTATE_SPEED / 10.0f);
				if (this->xas > XROTATE_SPEED) this->xas = XROTATE_SPEED;
			}

			if ( world->keys[VK_DOWN] )  {
				this->xas -= (XROTATE_SPEED / 10.0f);
				if (this->xas < -XROTATE_SPEED) this->xas = -XROTATE_SPEED;
			}
		
			if ( world->keys[VK_RIGHT] )	{
				this->yas += (YROTATE_SPEED / 10.0f);
				if (this->yas > YROTATE_SPEED) this->yas = YROTATE_SPEED;
			}
		
			if ( world->keys[VK_LEFT] )  {
				this->yas -= (YROTATE_SPEED / 10.0f);
				if (this->yas < -YROTATE_SPEED) this->yas = -YROTATE_SPEED;
			}

			break;

		case 1:

			if ( world->keys[VK_UP] )	{
				this->vxs += (XROTATE_SPEED / 10.0f);
				if (this->vxs > XROTATE_SPEED) this->vxs = XROTATE_SPEED;
			}

			if ( world->keys[VK_DOWN] )  {
				this->vxs -= (XROTATE_SPEED / 10.0f);
				if (this->vxs < -XROTATE_SPEED) this->vxs = -XROTATE_SPEED;
			}
		
			if ( world->keys[VK_RIGHT] )	{
				this->vys += (YROTATE_SPEED / 10.0f);
				if (this->vys > YROTATE_SPEED) this->vys = YROTATE_SPEED;
			}
		
			if ( world->keys[VK_LEFT] )  {
				this->vys -= (YROTATE_SPEED / 10.0f);
				if (this->vys < -YROTATE_SPEED) this->vys = -YROTATE_SPEED;
			}

			if (world->keys[VK_NEXT] && !world->keyp[VK_NEXT]) {
				this->view_id = world->next_object_id (this->view_id);
				world->keyp[VK_NEXT] = true;
			}
			if (!world->keys[VK_NEXT] && world->keyp[VK_NEXT]) world->keyp[VK_NEXT] = false;

			if (world->keys[VK_PRIOR] && !world->keyp[VK_PRIOR]) {
				this->view_id = world->prev_object_id (this->view_id);
				world->keyp[VK_PRIOR] = true;
			}
			if (!world->keys[VK_PRIOR] && world->keyp[VK_PRIOR]) world->keyp[VK_PRIOR] = false;

			if (world->keys[VK_SPACE] && !world->keyp[VK_SPACE]) {
				
				c_ball_p ball = world->get_object_by_id (this->view_id);

				if (ball) {

					float vx = -3.0 *  sin (this->wg2r(this->vya)) * cos (this->wg2r(this->vxa)); 
					float vy = -3.0 *  sin (this->wg2r(this->vxa));
					float vz = -3.0 *  cos (this->wg2r(this->vya)) * cos (this->wg2r(this->vxa));
					vp_t vector = {vx, vy, vz};

					ball->vector (vector);

				}

				world->keyp[VK_SPACE] = true;
			}
			if (!world->keys[VK_SPACE] && world->keyp[VK_SPACE]) world->keyp[VK_SPACE] = false;

			break;

	}

	// Rotate Inertia - X
	if ( !world->keys[VK_UP] && !world->keys[VK_DOWN] || this->view) {
		if (this->xas > (XROTATE_SPEED / 20.0f)) {
			this->xas -= (XROTATE_SPEED / 20.0f);
		} else if (this->xas < -(XROTATE_SPEED / 20.0f)) {
			this->xas += (XROTATE_SPEED / 20.0f);
		} else {
			this->xas = 0.0f;
		}
	}		

	// Rotate Inertia - Y
	if (!world->keys[VK_RIGHT] && !world->keys[VK_LEFT] || this->view) {
		if (this->yas > (YROTATE_SPEED / 20.0f)) {
			this->yas -= (YROTATE_SPEED / 20.0f);
		} else if (this->yas < -(YROTATE_SPEED / 20.0f)) {
			this->yas += (YROTATE_SPEED / 20.0f);
		} else {
			this->yas = 0.0f;
		}
	}

	// View Rotate Inertia - X
	if ( !world->keys[VK_UP] && !world->keys[VK_DOWN] || !this->view) {
		if (this->vxs > (XROTATE_SPEED / 20.0f)) {
			this->vxs -= (XROTATE_SPEED / 20.0f);
		} else if (this->xas < -(XROTATE_SPEED / 20.0f)) {
			this->vxs += (XROTATE_SPEED / 20.0f);
		} else {
			this->vxs = 0.0f;
		}
	}		

	// View Rotate Inertia - Y
	if (!world->keys[VK_RIGHT] && !world->keys[VK_LEFT] || !this->view) {
		if (this->vys > (YROTATE_SPEED / 20.0f)) {
			this->vys -= (YROTATE_SPEED / 20.0f);
		} else if (this->vys < -(YROTATE_SPEED / 20.0f)) {
			this->vys += (YROTATE_SPEED / 20.0f);
		} else {
			this->vys = 0.0f;
		}
	}

	return true;

}

void c_iface::look (void* vworld) {

	c_world_p world = (c_world_p) vworld;

	switch (this->view) {

		case 0:

			this->x = this->y = this->z = 0.0;

			this->xa += this->xas;
			this->ya += this->yas;

			if (this->ya < 0.0)		this->ya += 360.0;
			if (this->ya > 360.0)	this->ya -= 360.0;
			if (this->xa > 90.0)	this->xa =  90.0;
			if (this->xa < -90.0)	this->xa =  -90.0;

			this->cx = this->x + this->camera * sin (this->wg2r(this->ya)) * cos (this->wg2r(this->xa));
			this->cy = this->y + this->camera * sin (this->wg2r(this->xa));
			this->cz = this->z + this->camera * cos (this->wg2r(this->ya)) * cos (this->wg2r(this->xa));

			break;

		case 1:

			c_ball_p ball = world->get_object_by_id (this->view_id);

			if (ball == null) {
				this->view_id = world->next_object_id (this->view_id);
				c_ball_p ball = world->get_object_by_id (this->view_id);
			}

			if (ball == null) {
				this->view = 0;
				break;
			}

			this->x = ball->position.x;
			this->y = ball->position.y;
			this->z = ball->position.z;

			this->vxa += this->vxs;
			this->vya += this->vys;

			if (this->vya < 0.0)		this->vya += 360.0;
			if (this->vya > 360.0)		this->vya -= 360.0;
			if (this->vxa > 90.0)		this->vxa =  90.0;
			if (this->vxa < -90.0)		this->vxa =  -90.0;

			this->cx = this->x + this->camera * sin (this->wg2r(this->vya)) * cos (this->wg2r(this->vxa));
			this->cy = this->y + this->camera * sin (this->wg2r(this->vxa));
			this->cz = this->z + this->camera * cos (this->wg2r(this->vya)) * cos (this->wg2r(this->vxa));

			break;

	}

	// Look At Viewpoint
	gluLookAt (this->cx, this->cy, this->cz, this->x, this->y, this->z, 0, 1, 0);

}