/**

	3D Billiards Demo
	
	File			:	World Class Code

	Created by		:	Reznichenko Anton aka AlterVision
						mailto:me@av13.ru
						http://www.av13.ru

	Project started :	14 may 2009 - 12:27
	Last edited		:	18 may 2009 - 19:26

*/

#include <windows.h>
#include <math.h>
#include <stdlib.h>
#include <gl\gl.h>                      
#include <gl\glu.h>    
#include "world.h"

c_world::c_world() {

	for (int i = 0; i < 256; i++) {
		this->keyp[i] = 0;
		this->keys[i] = 0;
	}

	this->iface = new c_iface ();

	for (int i = 0; i < MAX_OBJECTS; i++) {
		this->objects[i] = null;
	}

	this->objects_count = COUNT_INC;
	this->objects_ptr = 1;
	this->create_object_ptr = 0;

	GLUquadricObj *quadratic;
	quadratic = gluNewQuadric();    
    gluQuadricNormals(quadratic, GLU_SMOOTH); 
	gluQuadricOrientation(quadratic, GLU_OUTSIDE);
	gluQuadricDrawStyle(quadratic, GLU_FILL);

	this->quadric = (void*) quadratic;

}

c_world::~c_world() {

	delete this->iface;

	for (int i = 0; i < MAX_OBJECTS; i++) {
		if (this->objects[i] != null) {
			delete this->objects[i];
		}
	}

}

void c_world::create () {

	// Counting World Bounds
	float min_x = -BOX_SIZE_X * 0.8;
	float max_x = 2.0 * BOX_SIZE_X * 0.8;
	float min_y = -BOX_SIZE_Y * 0.8;
	float max_y = 2.0 * BOX_SIZE_Y * 0.8;
	float min_z = -BOX_SIZE_Z * 0.8;
	float max_z = 2.0 * BOX_SIZE_Z * 0.8;

	// Creating Random Positioned Balls
	for (int i = 0; i < 10; i++) {

		// Random Position
		float x = min_x + max_x * rand() / (float) RAND_MAX;
		float y = min_y + max_y * rand() / (float) RAND_MAX;
		float z = min_z + max_z * rand() / (float) RAND_MAX;

		vp_t position = {x, y, z};

		// Calling Create Function
		this->create_object (position);

	}

}

int c_world::create_object(vp_t position) {

	int att = this->objects_count;
	int id = -1;

	while (att > 0) {
		this->create_object_ptr ++;
		if (this->create_object_ptr >= this->objects_count) this->create_object_ptr = 0;
		if (this->objects[this->create_object_ptr] == null) {
			id = create_object_ptr;
			break;
		}
		att--;
	}

	if (id == -1) {
		if (this->objects_count < MAX_OBJECTS) {
			this->create_object_ptr = this->objects_count + 1;
			id = this->create_object_ptr;
			this->objects_count += COUNT_INC;
			if (this->objects_count > MAX_OBJECTS) this->objects_count = MAX_OBJECTS;
		} else return -1;
	}
	
	this->objects[id] = new c_ball (id);
	this->objects[id]->create ((void*) this, position);

	return id;

}

int c_world::delete_object (int id) {

	c_ball_p obj = this->get_object_by_id (id);

	obj->destroy ((void*) this);

	delete this->objects[id];
	this->objects[id] = null;

	return true;

}

void c_world::process() {

	c_ball_p object;

	this->reset_objects ();

	// Processing Main Movements
	while ( object = this->get_next_object () ) {

		object->lock ();
		object->process( (void*) this );
		object->unlock ();

		if (object->active == false) {
			this->delete_object (object->id);
		}

	}

}

void c_world::collider () {

	// Processing Collisions
	for (int i = 0; i < this->objects_count; i++) {
		
		if (this->objects[i] == null) continue;

		c_ball_p ball1 = this->objects[i];
		ball1->lock ();

		for (int j = i + 1; j < this->objects_count; j++) {

			if (this->objects[j] == null) continue;

			c_ball_p ball2 = this->objects[j];
			ball2->lock ();

			// Distance Between Balls
			float distance = sqrt ( pow (ball1->position.x - ball2->position.x, 2) + pow (ball1->position.y - ball2->position.y, 2) + pow (ball1->position.z - ball2->position.z, 2) );

			// If Distance is LE Half o Size Summ
			if ( (2.0 * distance) <= (ball1->size + ball2->size) ) {
				
				// Collider ...
				ball2->speed = ball1->collide (ball2->position, ball2->speed, ball2->mass);

			}

			ball2->unlock ();

		}

		ball1->unlock ();

	}

}

void c_world::render() {

	// Making A Look
	this->iface->process ((void*) this);
	this->iface->look ((void*) this);

	c_ball_p object;

	this->reset_objects ();
	while (object = this->get_next_object()) {
		object->lock ();
		object->render ((void*) this);
		object->unlock ();
	}

	// Render The World Bounds

	glPolygonMode (GL_FRONT_AND_BACK, GL_LINE);

	glColor3f (1.0, 1.0, 1.0);
	glBegin (GL_QUADS);

		glVertex3f ( -BOX_SIZE_X,  BOX_SIZE_Y, -BOX_SIZE_Z);
		glVertex3f (  BOX_SIZE_X,  BOX_SIZE_Y, -BOX_SIZE_Z);
		glVertex3f (  BOX_SIZE_X,  BOX_SIZE_Y,  BOX_SIZE_Z);
		glVertex3f ( -BOX_SIZE_X,  BOX_SIZE_Y,  BOX_SIZE_Z);

		glVertex3f ( -BOX_SIZE_X, -BOX_SIZE_Y, -BOX_SIZE_Z);
		glVertex3f (  BOX_SIZE_X, -BOX_SIZE_Y, -BOX_SIZE_Z);
		glVertex3f (  BOX_SIZE_X, -BOX_SIZE_Y,  BOX_SIZE_Z);
		glVertex3f ( -BOX_SIZE_X, -BOX_SIZE_Y,  BOX_SIZE_Z);

		glVertex3f ( -BOX_SIZE_X, -BOX_SIZE_Y,  BOX_SIZE_Z);
		glVertex3f (  BOX_SIZE_X, -BOX_SIZE_Y,  BOX_SIZE_Z);
		glVertex3f (  BOX_SIZE_X,  BOX_SIZE_Y,  BOX_SIZE_Z);
		glVertex3f ( -BOX_SIZE_X,  BOX_SIZE_Y,  BOX_SIZE_Z);

		glVertex3f ( -BOX_SIZE_X, -BOX_SIZE_Y, -BOX_SIZE_Z);
		glVertex3f (  BOX_SIZE_X, -BOX_SIZE_Y, -BOX_SIZE_Z);
		glVertex3f (  BOX_SIZE_X,  BOX_SIZE_Y, -BOX_SIZE_Z);
		glVertex3f ( -BOX_SIZE_X,  BOX_SIZE_Y, -BOX_SIZE_Z);

		glVertex3f (  BOX_SIZE_X, -BOX_SIZE_Y, -BOX_SIZE_Z);
		glVertex3f (  BOX_SIZE_X, -BOX_SIZE_Y,  BOX_SIZE_Z);
		glVertex3f (  BOX_SIZE_X,  BOX_SIZE_Y,  BOX_SIZE_Z);
		glVertex3f (  BOX_SIZE_X,  BOX_SIZE_Y, -BOX_SIZE_Z);

		glVertex3f ( -BOX_SIZE_X, -BOX_SIZE_Y, -BOX_SIZE_Z);
		glVertex3f ( -BOX_SIZE_X, -BOX_SIZE_Y,  BOX_SIZE_Z);
		glVertex3f ( -BOX_SIZE_X,  BOX_SIZE_Y,  BOX_SIZE_Z);
		glVertex3f ( -BOX_SIZE_X,  BOX_SIZE_Y, -BOX_SIZE_Z);

	glEnd ();

	glPolygonMode (GL_FRONT_AND_BACK, GL_FILL);

}

void c_world::destroy() {

}

// Resetting Object Pointer To Beginnibg
void c_world::reset_objects () {

	this->objects_ptr = -1;

}

// Getting Next Object
c_ball_p c_world::get_next_object () {

	do {
		this->objects_ptr ++;
		if (this->objects_ptr >= this->objects_count) {
			this->reset_objects ();
			return null;
		}
	} while (this->objects[this->objects_ptr] == null);

	return this->objects[this->objects_ptr];

}

// Getting Object By Id
c_ball_p c_world::get_object_by_id(int id) {

	if (id < 0 || id > this->objects_count) return null;

	// Returning Object or Null (if object id undefined)
	return this->objects[id];

}

// Get Next Object ID
int c_world::next_object_id (int id) {

	int attempts = this->objects_count;

	while (attempts) {
		attempts --;
		id ++;
		if (id >= this->objects_count) id = 0;
		if (this->objects[id]) return id;
	} 

	return id;

}

// Get Previous Object ID
int c_world::prev_object_id (int id) {

	int attempts = this->objects_count;

	while (attempts) {
		attempts --;
		id --;
		if (id < 0) id = this->objects_count - 1;
		if (this->objects[id]) return id;
	} 

	return id;

}
