/**

	3D Billiards Demo
	
	File			:	Main 

	Created by		:	Reznichenko Anton aka AlterVision
						mailto:me@av13.ru
						http://www.av13.ru

	Project started :	14 may 2009 - 12:27
	Last edited		:	18 may 2009 - 19:24

	Description:

		This is really simple physics simulator demo for real-time systems.
		
		This small demo can show you, how to make programms using real-time
		thread syncronization. Here we have three-threads program with
		quite simple syncronization using times and mutex.

		Additional Information:
			Idea:	http://www.me.av13.ru/arts/physical-simulator/
			Code:	http://www.me.av13.ru/arts/physical-simulator/#code
			Files:	http://www.me.av13.ru/arts/physical-simulator/#files

		PS: the .product will make you .happy %)

	License				:	GNU GLP
	License notice		:	Please, do not use this product! It can cause 
							possible injures and brain problems!
							I told you!!						

*/

#include "game/world.h"

#include <windows.h>          
#include <process.h>
#include <math.h>
#include <gl\gl.h>                      
#include <gl\glu.h>                    


HDC		hDC = null;      // GDI Context
HGLRC	hRC = null;      // Render Context
HWND	hWnd = null;     // Window Descriptor
HINSTANCE hInstance;    // App Descriptor
int		work_status;

c_world_p world;		// global context

void init_gl_scene (GLsizei Width, GLsizei Height) {
	
	srand((int) GetTickCount());

	glClearColor(0.2f, 0.2f, 0.2f, 1.0f); // Background color
	glClearDepth(1.0f);       

	glEnable(GL_DEPTH_TEST);     
	glDepthFunc(GL_LEQUAL);     
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST); 

	glPolygonMode (GL_FRONT_AND_BACK, GL_FILL);


	GLfloat LightAmbient[]	= { 0.4f, 0.4f, 0.4f, 1.0f };     
	GLfloat LightDiffuse[]	= { 0.9f, 0.9f, 0.9f, 1.0f };
	GLfloat LightSpecular[]	= { 1.0f, 1.0f, 1.0f, 1.0f };
	GLfloat LightSpotDir[]	= { -1.0f, -1.0f, -1.0f };
	GLfloat LightPosition[] = { 10.0f, 10.0f, 10.0f, 1.0f };    

	glEnable (GL_LIGHTING);
	glLightfv (GL_LIGHT1, GL_AMBIENT,  LightAmbient);	// Ambient Lighting
	glLightfv (GL_LIGHT1, GL_DIFFUSE,  LightDiffuse);	// Diffuse Lighting
	glLightfv (GL_LIGHT1, GL_SPECULAR, LightSpecular);	// Diffuse Lighting
	glLightfv (GL_LIGHT1, GL_POSITION, LightPosition);	// Light Position
	glLightfv (GL_LIGHT1, GL_SPOT_DIRECTION, LightSpotDir);
	glLightf  (GL_LIGHT1, GL_SPOT_EXPONENT, 0.0f);
	glLightf  (GL_LIGHT1, GL_SPOT_CUTOFF, 180.0f);
	glLightf  (GL_LIGHT1, GL_CONSTANT_ATTENUATION, 1.0f);
	glLightf  (GL_LIGHT1, GL_LINEAR_ATTENUATION, 0.0f);
	glLightf  (GL_LIGHT1, GL_QUADRATIC_ATTENUATION, 0.0f);
	glEnable  (GL_LIGHT1);								// Fiat Lux!

	ShowCursor(false);

	world->create ();

}

void resize_gl_scene (GLsizei Width, GLsizei Height) {

	if (Height==0)  Height=1;			// AntiDivByZero
	
	glViewport(0, 0, Width, Height);	// Reset Perspective

	glMatrixMode(GL_PROJECTION);        // Reset projection maxtix
    glLoadIdentity();                   // Reset projection maxtix

    // Counting window size
	gluPerspective (45.0f, (GLfloat) Width / (GLfloat) Height, 0.0f, 150.0f);
    glMatrixMode(GL_MODELVIEW);			// Choosing model viewing matrix

	world->win_cx = world->win_x + (world->screen_x / 2);
	world->win_cy = world->win_y + (world->screen_y / 2);

}

void free_gl_scene () {

	ChangeDisplaySettings(NULL, 0);

	wglMakeCurrent(hDC,NULL);
	wglDeleteContext(hRC);
	ReleaseDC(hWnd,hDC);

	work_status = false;
	Sleep (1000);

	world->destroy ();

	// Appocalipsis ;)
	delete world;

}

void __cdecl thread_process ( void *param ) {

	while (work_status) {

		if (world == null) break;

		DWORD start_time = GetTickCount();
			
			world->process ();

		while((GetTickCount() - start_time) < DRAW_TIMEOUT) Sleep(2);

	}

	_endthread ();
	
}

void __cdecl thread_collider ( void *param ) {

	while (work_status) {

		if (world == null) break;

		DWORD start_time = GetTickCount();

			world->collider ();

		while((GetTickCount() - start_time) < DRAW_TIMEOUT) Sleep(2);

	}

	_endthread ();

}

LRESULT CALLBACK WndProc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {

	RECT    Screen;         //     
	GLuint  PixelFormat;
	static  PIXELFORMATDESCRIPTOR pfd =	{
		sizeof(PIXELFORMATDESCRIPTOR),  //   
        1,                              //   (?)
        PFD_DRAW_TO_WINDOW |            //   
        PFD_SUPPORT_OPENGL |            //   OpenGL
		PFD_GENERIC_ACCELERATED |		// -  ,  ,      ...
		PFD_DOUBLEBUFFER,               //    
        PFD_TYPE_RGBA,                  //  RGBA 
        32,                             //  16   
        0, 0, 0, 0, 0, 0,               //    (?)
        0,                              //   
        0,                              //    (?)
        0,                              //   
        0, 0, 0, 0,                     //    (?)
        32,                             // 16  Z- ( ) 
        0,                              //   
        0,                              //    (?)
        PFD_MAIN_PLANE,                 //   
        0,                              //  (?)
        0, 0, 0                         //    (?)
	};

	switch (message)        // Main Switch
	{

		// App Create
        case WM_CREATE:

			// WM_Create :: Start

			hDC = GetDC (hWnd);      
			PixelFormat = ChoosePixelFormat(hDC, &pfd);
			if (!PixelFormat){
				MessageBox(0, "Can't Find A Suitable PixelFormat.", "Error", MB_OK | MB_ICONERROR);
				PostQuitMessage(0);
                break; 
            }

			SetPixelFormat (hDC, PixelFormat, &pfd);

			hRC = wglCreateContext (hDC);
			if(!hRC) {
                MessageBox(0, "Can't Create A GL Rendering Context.", "Error", MB_OK | MB_ICONERROR);
                PostQuitMessage(0);
                break;
            }

			if(!wglMakeCurrent(hDC, hRC)) {
                MessageBox(0, "Can't activate GLRC.", "Error", MB_OK | MB_ICONERROR);
                PostQuitMessage(0);
                break;
            }

			GetClientRect(hWnd, &Screen);
			init_gl_scene (Screen.right, Screen.bottom);
			break;
			// WM_Create :: End

		// App Die
		case WM_DESTROY:
		case WM_CLOSE:

			// WM_Close :: Start
			free_gl_scene ();
			PostQuitMessage(0);
			break;
			// WM_Close :: End

		// App KeyState

		case WM_KEYDOWN:
			world->keys[wParam] = true;
			break;

		case WM_KEYUP:
			world->keys[wParam] = false;
			break;

		case WM_MOUSEMOVE:
			world->mmx	= world->win_cx - LOWORD(lParam) - world->win_x;
			world->mmy	= world->win_cy - HIWORD(lParam) - world->win_y;
			break;

		// Win Resize

		case WM_SIZE:
			resize_gl_scene (LOWORD(lParam),HIWORD(lParam));
			break;

		// Default Win Proc		

        default:
			return (DefWindowProc(hWnd, message, wParam, lParam));
    } // end - Main Switch

	return (0);

}

int WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance, LPSTR lpCmdLine,int nCmdShow)
{
	MSG             msg;    //   Windows
	WNDCLASS        wc;		//   Windows    
	HWND            hWnd;   //   

	wc.style                = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wc.lpfnWndProc          = (WNDPROC) WndProc;
	wc.cbClsExtra           = 0;
	wc.cbWndExtra           = 0;
	wc.hInstance            = hInstance;
	wc.hIcon                = NULL;
	wc.hCursor              = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground        = NULL;
	wc.lpszMenuName         = NULL;
	wc.lpszClassName        = "OpenGL WinClass";

	if(!RegisterClass(&wc))	{
		MessageBox (0,"Failed To Register The Window Class.", "Error", MB_OK|MB_ICONERROR);
		return false;
	}

	world = new c_world_t ();

	world->win_x = 0;
	world->win_y = 0;
	world->screen_x = GetSystemMetrics(SM_CXSCREEN);
	world->screen_y = GetSystemMetrics(SM_CYSCREEN);
	world->win_cx = world->win_x + (world->screen_x / 2);
	world->win_cy = world->win_y + (world->screen_y / 2);

	hWnd = CreateWindow (
        "OpenGL WinClass",
        "Battle Of The Spheres - AlterVision Creatives",	//   
        WS_POPUP | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
        world->win_x, world->win_y,						//    
        world->screen_x, world->screen_y,				//    
        NULL,
        NULL,
        hInstance,
        NULL
	);

	if(!hWnd) {
		MessageBox(0,"Window Creation Error.", "Error", MB_OK|MB_ICONERROR); 
		return false;
	}

	ShowWindow(hWnd, SW_SHOW);
	UpdateWindow(hWnd);
	SetFocus(hWnd);

	// Preparing To Work
	work_status = true;

	// Starting Threads
	_beginthread ( thread_process,  null, null );
	_beginthread ( thread_collider, null, null );

	while (1)
	{
		// Sync Rate :: 30 fps :: Get Time
		DWORD start_time = GetTickCount();

		while (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))	{
			if (GetMessage(&msg, NULL, 0, 0)) {
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			} else {
				return true;
			}
		}

		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glLoadIdentity();
		world->render ();
		glFlush();
		SwapBuffers(hDC);    

		// EscapeKey for Exit Procedure
		if (world->keys[VK_ESCAPE]) SendMessage (hWnd,WM_CLOSE,0,0);    

		// Sync Rate :: 30 fps :: Syncronize;
		while((GetTickCount() - start_time) < DRAW_TIMEOUT) Sleep(3);
	}

}

